@@ -237,6 +237,7 @@ private:
 	CachedFileStat cstat;
 	WatchdogLauncher watchdogLauncher;
 	boost::mutex cstatMutex;
+	boost::mutex configMutex;
 
 	static Json::Value strsetToJson(const set<string> &input) {
 		Json::Value result(Json::arrayValue);
@@ -446,9 +447,11 @@ private:
 		TRACE_POINT();
 
 		DirectoryMapper mapper(r, config, wrapperRegistry, &cstat,
-			&cstatMutex, serverConfig.statThrottleRate);
+		    &cstatMutex, serverConfig.statThrottleRate, &configMutex);
 		try {
-			if (mapper.getDetectorResult().isNull()) {
+			if (config->getAppStartCommand().empty()
+			 && mapper.getDetectorResult().isNull())
+			{
 				// (B) is not true.
 				disableRequestNote(r);
 				return false;
@@ -988,8 +991,16 @@ private:
 		// Phusion Passenger options.
 		addHeader(result, P_STATIC_STRING("!~PASSENGER_APP_ROOT"),
 			mapper.getAppRoot());
-		addHeader(result, P_STATIC_STRING("!~PASSENGER_APP_TYPE"),
-			mapper.getDetectorResult().wrapperRegistryEntry->language);
+		if (!config->getAppStartCommand().empty()) {
+			addHeader(result, P_STATIC_STRING("!~PASSENGER_APP_START_COMMAND"),
+				config->getAppStartCommand());
+		} else if (mapper.getDetectorResult().wrapperRegistryEntry != NULL) {
+			addHeader(result, P_STATIC_STRING("!~PASSENGER_APP_TYPE"),
+				mapper.getDetectorResult().wrapperRegistryEntry->language);
+		} else {
+			addHeader(result, P_STATIC_STRING("!~PASSENGER_APP_START_COMMAND"),
+				mapper.getDetectorResult().appStartCommand);
+		}
 		constructRequestHeaders_autoGenerated(r, config, result);
 
 		/*********************/
@@ -1323,6 +1334,7 @@ public:
 		config["controller_socket_backlog"] = serverConfig.socketBacklog;
 		config["controller_file_buffered_channel_buffer_dir"] = nonEmptyString(serverConfig.dataBufferDir);
 		config["instance_registry_dir"] = nonEmptyString(serverConfig.instanceRegistryDir);
+		config["spawn_dir"] = nonEmptyString(serverConfig.spawnDir);
 		config["security_update_checker_disabled"] = serverConfig.disableSecurityUpdateCheck;
 		config["security_update_checker_proxy_url"] = nonEmptyString(serverConfig.securityUpdateCheckProxy);
 		config["telemetry_collector_disabled"] = serverConfig.disableAnonymousTelemetry;
@@ -1343,6 +1355,7 @@ public:
 		config["admin_panel_auth_type"] = nonEmptyString(serverConfig.adminPanelAuthType);
 		config["admin_panel_username"] = nonEmptyString(serverConfig.adminPanelUsername);
 		config["admin_panel_password"] = nonEmptyString(serverConfig.adminPanelPassword);
+		config["disable_log_prefix"] = serverConfig.disableLogPrefix;
 
 		if (!serverConfig.logFile.empty()) {
 			config["log_target"] = serverConfig.logFile.toString();
